package org.foxarmy.barcodescannerforemployees

import android.content.ContentValues
import android.content.Context
import android.database.DatabaseUtils
import android.database.sqlite.SQLiteDatabase
import android.database.sqlite.SQLiteOpenHelper
import android.provider.BaseColumns
import android.widget.Toast
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKeys
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import org.foxarmy.barcodescannerforemployees.dataclasses.Category
import org.foxarmy.barcodescannerforemployees.dataclasses.Product
import java.io.File

object AbstractProductContract {
    object AbstractProductEntry : BaseColumns {
        const val TABLE_NAME = "abstract_products"
        const val BARCODE = "barcode"
        const val PRODUCT_NAME = "name"
        const val PRODUCT_NET_WEIGHT = "net_weight"
        const val IMAGE_FILENAME = "image_filename"
        const val CATEGORY = "category"
        const val UNIT = "unit"
    }
}

object CategoriesContract {
    object CategoryEntry : BaseColumns {
        const val TABLE_NAME = "categories"
        const val CATEGORY_NAME = "category_name"
    }
}

object ProductContract {
    object ProductEntry : BaseColumns {
        const val TABLE_NAME = "products"
        const val ABSTRACT_PRODUCT_ID = "abstract_product_id"
        const val AMOUNT = "amount"
        const val DATE_OF_PRODUCTION = "date_of_production"
        const val EXPIRY_DATE = "expiry_date"
    }
}

const val SQL_CREATE_ABSTRACT_PRODUCTS_TABLE =
    "CREATE TABLE ${AbstractProductContract.AbstractProductEntry.TABLE_NAME} (" +
            "${BaseColumns._ID} INTEGER PRIMARY KEY," +
            "${AbstractProductContract.AbstractProductEntry.BARCODE} TEXT," +
            "${AbstractProductContract.AbstractProductEntry.PRODUCT_NAME} TEXT," +
            "${AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT} REAL," +
            "${AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME} TEXT," +
            "${AbstractProductContract.AbstractProductEntry.CATEGORY} INTEGER," +
            "${AbstractProductContract.AbstractProductEntry.UNIT} INTEGER)"

const val SQL_CREATE_PRODUCTS_TABLE =
    "CREATE TABLE ${ProductContract.ProductEntry.TABLE_NAME} (" +
            "${BaseColumns._ID} INTEGER PRIMARY KEY," +
            "${ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID} INTEGER," +
            "${ProductContract.ProductEntry.AMOUNT} INTEGER," +
            "${ProductContract.ProductEntry.DATE_OF_PRODUCTION} INTEGER," +
            "${ProductContract.ProductEntry.EXPIRY_DATE} INTGER)"

const val SQL_CREATE_CATEGORIES_TABLE =
    "CREATE TABLE ${CategoriesContract.CategoryEntry.TABLE_NAME} (" +
            "${BaseColumns._ID} INTEGER PRIMARY KEY," +
            "${CategoriesContract.CategoryEntry.CATEGORY_NAME} TEXT)"

class DBStorageController(context: Context, dbname: String) :
    SQLiteOpenHelper(context, dbname, null, DATABASE_VERSION) {

    private lateinit var dbname: String
    private lateinit var db: SQLiteDatabase

    override fun onCreate(db: SQLiteDatabase) {
        this.db = db
        DATABASE_NAME = "$dbname.db"

        db.execSQL(SQL_CREATE_ABSTRACT_PRODUCTS_TABLE)
        db.execSQL(SQL_CREATE_PRODUCTS_TABLE)
        db.execSQL(SQL_CREATE_CATEGORIES_TABLE)
    }

    override fun onUpgrade(db: SQLiteDatabase?, oldV: Int, newV: Int) {
        TODO("Not yet implemented")
    }

    fun eraseCategory(id: Int, context: Context) {
        val productsInCategory = getAllAbstractProductInCategory(id)

        for (product in productsInCategory.iterator()) {
            eraseAbstractProduct(product.id, context)
        }

        db.delete(CategoriesContract.CategoryEntry.TABLE_NAME, BaseColumns._ID + "=" + id, null)
    }

    fun getCategoryNameById(id: Int): String {
        var result = ""

        val projection = arrayOf(CategoriesContract.CategoryEntry.CATEGORY_NAME)
        val selection = "${BaseColumns._ID} = ?"
        val selectionArgs = arrayOf(id.toString())

        val cursor = db.query(
            CategoriesContract.CategoryEntry.TABLE_NAME,
            projection,
            selection,
            selectionArgs,
            null,
            null,
            null
        )

        with(cursor) {
            while (moveToNext()) {
                result = getString(getColumnIndexOrThrow(CategoriesContract.CategoryEntry.CATEGORY_NAME))
            }
        }

        return result
    }

    fun getAllAbstractProductInCategory(id: Int): List<AbstractProduct> {

        var result = mutableListOf<AbstractProduct>()
        val projection = arrayOf(
            BaseColumns._ID,
            AbstractProductContract.AbstractProductEntry.BARCODE,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NAME,
            AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT,
            AbstractProductContract.AbstractProductEntry.UNIT
        )

        val selection = "${AbstractProductContract.AbstractProductEntry.CATEGORY} = ?"
        val selectionArgs = arrayOf(id.toString())

        val cursor = db.query(
            AbstractProductContract.AbstractProductEntry.TABLE_NAME,
            projection,
            selection,
            selectionArgs,
            null,
            null,
            null
        )

        with(cursor) {
            while (moveToNext()) {
                val abstractProductId = getInt(getColumnIndexOrThrow(BaseColumns._ID))
                val abstractProductBarcode =
                    getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.BARCODE))
                val abstractProductName =
                    getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NAME))
                val abstractProductNetWeight =
                    getDouble(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT))
                val abstractProductImageHash =
                    getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME))
                val abstractProductUnit =
                    getInt(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.UNIT))

                val abstractProduct = AbstractProduct(
                    abstractProductId,
                    abstractProductBarcode,
                    abstractProductName,
                    abstractProductNetWeight,
                    abstractProductImageHash,
                    category = id,
                    abstractProductUnit
                )

                result.add(abstractProduct)
            }
        }

        return result
    }

    fun getAmountOfAbstractProductsInCategory(id: Int): Int {
        return DatabaseUtils.longForQuery(
            db,
            "SELECT COUNT(*) FROM ${AbstractProductContract.AbstractProductEntry.TABLE_NAME} WHERE ${AbstractProductContract.AbstractProductEntry.CATEGORY} = ?",
            arrayOf(id.toString())
        ).toInt()
    }

    fun eraseAbstractProduct(id: Int, context: Context) {
        val projectionForAbstractProduct = arrayOf(
            AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME
        )

        val selectionForAbstractProduct = "${BaseColumns._ID} = ?"
        val selectionArgsForAbstractProduct = arrayOf(id.toString())
        val cursorForAbstractProduct = db.query(
            AbstractProductContract.AbstractProductEntry.TABLE_NAME,
            projectionForAbstractProduct,
            selectionForAbstractProduct,
            selectionArgsForAbstractProduct,
            null,
            null,
            null
        )
        var imageHash: String = ""

        with(cursorForAbstractProduct) {
            while (moveToNext()) {
                val productImageHash =
                    getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME))
                imageHash = productImageHash
            }
        }
        val picturesDir = File(context.filesDir, "pictures")
        val thumbnailsDir = File(context.cacheDir, "thumbnails")

        File(picturesDir, "$imageHash.png").delete()
        File(thumbnailsDir, "$imageHash.webp").delete()

        val projectionForProducts = arrayOf(BaseColumns._ID)
        val selectionForProducts = "${ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID} = ?"
        val selectionArgsForProducts = arrayOf(id.toString())

        val cursorForProducts = db.query(
            ProductContract.ProductEntry.TABLE_NAME,
            projectionForProducts,
            selectionForProducts,
            selectionArgsForProducts,
            null,
            null,
            null
        )

        with(cursorForProducts) {
            while (moveToNext()) {
                eraseProduct(getInt(getColumnIndexOrThrow(BaseColumns._ID)))
            }
        }

        db.delete(AbstractProductContract.AbstractProductEntry.TABLE_NAME, BaseColumns._ID + "=" + id, null)
    }

    fun eraseProduct(id: Int) {
        db.delete(ProductContract.ProductEntry.TABLE_NAME, BaseColumns._ID + "=" + id, null)
    }

    fun insertNewProduct(product: Product) {
        val values = ContentValues().apply {
            put(ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID, product.abstractProductId)
            put(ProductContract.ProductEntry.AMOUNT, product.amount)
            put(ProductContract.ProductEntry.DATE_OF_PRODUCTION, product.dateOfProduction)
            put(ProductContract.ProductEntry.EXPIRY_DATE, product.dateOfExpiry)
        }

        db.insert(ProductContract.ProductEntry.TABLE_NAME, null, values)
    }

    fun findAmountOfProductsWithExpiryDate(date: Long): Int {
        var amount = 0

        val projection = arrayOf(
            ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID,
            ProductContract.ProductEntry.AMOUNT,
            ProductContract.ProductEntry.DATE_OF_PRODUCTION,
        )

        val selection = "${ProductContract.ProductEntry.EXPIRY_DATE} = ?"
        val selectionArgs = arrayOf(date.toString())

        val cursor =
            db.query(ProductContract.ProductEntry.TABLE_NAME, projection, selection, selectionArgs, null, null, null)

        with(cursor) {
            while (moveToNext()) {
                amount++
            }
        }

        return amount
    }

    fun findAllExpiryDates(): Set<Long> {
        val dates: MutableSet<Long> = mutableSetOf<Long>()

        val projection = arrayOf(
            ProductContract.ProductEntry.EXPIRY_DATE
        )

        val orderBy = "${ProductContract.ProductEntry.EXPIRY_DATE} ASC"

        val cursor = db.query(ProductContract.ProductEntry.TABLE_NAME, projection, null, null, null, null, orderBy)

        with(cursor) {
            while (moveToNext()) {
                dates.add(getLong(getColumnIndexOrThrow(ProductContract.ProductEntry.EXPIRY_DATE)))
            }
        }

        return dates
    }

    fun findAbstractProductByBarcode(barcode: String): AbstractProduct? {
        var abstractProduct: AbstractProduct? = null
        val projection = arrayOf(
            BaseColumns._ID,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NAME,
            AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT,
            AbstractProductContract.AbstractProductEntry.CATEGORY,
            AbstractProductContract.AbstractProductEntry.UNIT
        )

        val selection = "${AbstractProductContract.AbstractProductEntry.BARCODE} = ?"
        val selectionArgs = arrayOf(barcode)

        val cursor = db.query(
            AbstractProductContract.AbstractProductEntry.TABLE_NAME,
            projection,
            selection,
            selectionArgs,
            null,
            null,
            null
        )

        with(cursor) {
            while (moveToNext()) {
                val id = getInt(getColumnIndexOrThrow(BaseColumns._ID))
                val productName =
                    getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NAME))
                val imageFilename =
                    getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME))
                val netWeight =
                    getDouble(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT))
                val category = getInt(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.CATEGORY))
                val unit = getInt(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.UNIT))

                abstractProduct = AbstractProduct(id, barcode, productName, netWeight, imageFilename, category, unit)
            }
        }

        return abstractProduct
    }

    fun findAbstractProductById(id: Int): AbstractProduct? {
        var abstractProduct: AbstractProduct? = null
        val projection = arrayOf(
            BaseColumns._ID,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NAME,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT,
            AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME,
            AbstractProductContract.AbstractProductEntry.CATEGORY,
            AbstractProductContract.AbstractProductEntry.BARCODE,
            AbstractProductContract.AbstractProductEntry.UNIT
        )

        val selection = "${BaseColumns._ID} = ?"
        val selectionArgs = arrayOf(id.toString())

        val cursor = db.query(
            AbstractProductContract.AbstractProductEntry.TABLE_NAME,
            projection,
            selection,
            selectionArgs,
            null,
            null,
            null
        )

        with(cursor) {
            while (moveToNext()) {
                val productId = getInt(getColumnIndexOrThrow(BaseColumns._ID))
                val barcode = getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.BARCODE))
                val name = getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NAME))
                val netWeight =
                    getDouble(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT))
                val imageHash =
                    getString((getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME)))
                val category = getInt(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.CATEGORY))
                val unit = getInt(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.UNIT))

                abstractProduct = AbstractProduct(productId, barcode, name, netWeight, imageHash, category, unit)
            }
        }

        return abstractProduct
    }

    fun updateProduct(product: Product) {
        val values = ContentValues().apply {
            put(ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID, product.abstractProductId)
            put(ProductContract.ProductEntry.AMOUNT, product.amount)
            put(ProductContract.ProductEntry.DATE_OF_PRODUCTION, product.dateOfProduction)
            put(ProductContract.ProductEntry.EXPIRY_DATE, product.dateOfExpiry)
        }
        db.update(
            ProductContract.ProductEntry.TABLE_NAME,
            values,
            "${BaseColumns._ID} = ?",
            arrayOf(product.id.toString())
        )
    }

    fun addAbstractProduct(abstractProduct: AbstractProduct, context: Context) {
        val values = ContentValues().apply {
            put(AbstractProductContract.AbstractProductEntry.BARCODE, abstractProduct.barcode)
            put(AbstractProductContract.AbstractProductEntry.PRODUCT_NAME, abstractProduct.name)
            put(AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT, abstractProduct.netWeight.toString())
            put(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME, abstractProduct.imageHash)
            put(AbstractProductContract.AbstractProductEntry.CATEGORY, abstractProduct.category)
            put(AbstractProductContract.AbstractProductEntry.UNIT, abstractProduct.unit)
        }

        val id = db.insert(AbstractProductContract.AbstractProductEntry.TABLE_NAME, null, values)
        val n = Net()
        val masterKeyAlias = MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC)

        val sharedPreferences = EncryptedSharedPreferences.create(
            "sensitive",
            masterKeyAlias,
            context,
            EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
            EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
        )

        n.language = sharedPreferences.getString("language", "en-US")!!
        n.server = sharedPreferences.getString("server", "")!!
        n.token = sharedPreferences.getString("token", "")!!
        val pictureFile = File(File(context.filesDir, "pictures"), "${abstractProduct.imageHash}.png")
        val response = n.uploadAbstractProduct(1, abstractProduct, pictureFile);

        Toast.makeText(context, response.body!!.string(), Toast.LENGTH_LONG).show()
    }

    fun updateAbstractProduct(abstractProduct: AbstractProduct, context: Context) {
        val values = ContentValues().apply {
            put(AbstractProductContract.AbstractProductEntry.BARCODE, abstractProduct.barcode)
            put(AbstractProductContract.AbstractProductEntry.PRODUCT_NAME, abstractProduct.name)
            put(AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT, abstractProduct.netWeight.toString())
            put(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME, abstractProduct.imageHash)
            put(AbstractProductContract.AbstractProductEntry.CATEGORY, abstractProduct.category)
            put(AbstractProductContract.AbstractProductEntry.UNIT, abstractProduct.unit)
        }

        db.update(
            AbstractProductContract.AbstractProductEntry.TABLE_NAME,
            values,
            "${BaseColumns._ID} = ?",
            arrayOf(abstractProduct.id.toString())
        )
    }

    fun getAllCategories(): List<Category> {
        val categories = mutableListOf<Category>()

        val projection = arrayOf(
            CategoriesContract.CategoryEntry.CATEGORY_NAME
        )

        val cursor = db.query(
            CategoriesContract.CategoryEntry.TABLE_NAME,
            projection,
            null,
            null,
            null,
            null,
            BaseColumns._ID + " ASC"
        )

        with(cursor) {
            while (moveToNext()) {
                val category = Category(
                    getInt(getColumnIndexOrThrow(BaseColumns._ID)),
                    getString(getColumnIndexOrThrow(CategoriesContract.CategoryEntry.CATEGORY_NAME))
                )
                categories.add(category)
            }
        }

        return categories
    }

    fun addCategory(category: Category) {
        val values = ContentValues().apply {
            put(CategoriesContract.CategoryEntry.CATEGORY_NAME, category.name)
        }

        db.insert(CategoriesContract.CategoryEntry.TABLE_NAME, null, values)
    }

    fun updateCategory(category: Category) {
        val values = ContentValues().apply {
            put(CategoriesContract.CategoryEntry.CATEGORY_NAME, category.name)
        }
        db.update(CategoriesContract.CategoryEntry.TABLE_NAME, values, "${BaseColumns._ID} = ?", arrayOf(category.id.toString()))
    }

    fun getSortedListOfProducts(selectedSort: Int, filterBy: String, filter: String): List<Product> {
        val products = mutableListOf<Product>()

        val projection = arrayOf(
            BaseColumns._ID,
            ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID,
            ProductContract.ProductEntry.AMOUNT,
            ProductContract.ProductEntry.DATE_OF_PRODUCTION,
            ProductContract.ProductEntry.EXPIRY_DATE,
        )

        var orderBy: String = ""

        when (selectedSort) {
            0 -> {
                orderBy =
                    "(SELECT ${AbstractProductContract.AbstractProductEntry.PRODUCT_NAME} FROM ${AbstractProductContract.AbstractProductEntry.TABLE_NAME} WHERE ${BaseColumns._ID} = ${ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID}) ASC"
            }

            1 -> {
                orderBy =
                    "(SELECT ${AbstractProductContract.AbstractProductEntry.CATEGORY} FROM ${AbstractProductContract.AbstractProductEntry.TABLE_NAME} WHERE ${BaseColumns._ID} = ${ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID}) ASC"
            }

            3 -> {
                orderBy = "${ProductContract.ProductEntry.DATE_OF_PRODUCTION} ASC"
            }

            4 -> {
                orderBy = "${ProductContract.ProductEntry.EXPIRY_DATE} ASC"
            }
        }

        var selection: String? = null
        var selectionArgs: Array<String>? = null

        when (filterBy) {
            "expiryDate" -> {
                selection = "${ProductContract.ProductEntry.EXPIRY_DATE} = ?"
                selectionArgs = arrayOf(filter)
            }

            "" -> {}
        }

        val cursor = db.query(ProductContract.ProductEntry.TABLE_NAME, projection, selection, selectionArgs, null, null, orderBy)

        with(cursor) {
            while (moveToNext()) {
                val productId = getInt(getColumnIndexOrThrow(BaseColumns._ID))
                val abstractProductId =
                    getInt(getColumnIndexOrThrow(ProductContract.ProductEntry.ABSTRACT_PRODUCT_ID))
                val amount = getInt(getColumnIndexOrThrow(ProductContract.ProductEntry.AMOUNT))
                val dateOfProduction =
                    getLong(getColumnIndexOrThrow(ProductContract.ProductEntry.DATE_OF_PRODUCTION))
                val dateOfExpiry = getLong(getColumnIndexOrThrow(ProductContract.ProductEntry.EXPIRY_DATE))

                val product = Product(productId, abstractProductId, amount, dateOfProduction, dateOfExpiry)

                if (selectedSort == 2) { //freshness
                    products.add(product)
                }
            }
        }

        products.sortWith(compareByDescending { it.freshness })

        return products
    }

    fun getSortedListOfAbstractProducts(selectedSort: Int, filterBy: String, filter: Array<String>): List<AbstractProduct> {

        val abstractProducts = mutableListOf<AbstractProduct>()

        val projection = arrayOf(
            BaseColumns._ID,
            AbstractProductContract.AbstractProductEntry.BARCODE,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NAME,
            AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT,
            AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME,
            AbstractProductContract.AbstractProductEntry.CATEGORY,
            AbstractProductContract.AbstractProductEntry.UNIT
        )

        var orderBy: String = ""
        when(selectedSort) {
            0 -> {
                orderBy = "${AbstractProductContract.AbstractProductEntry.PRODUCT_NAME} ASC"
            }
            1 -> {
                orderBy = "${AbstractProductContract.AbstractProductEntry.CATEGORY} ASC"
            }
        }

        var selection = ""
        var selectionArgs: Array<String>? = null

        when (filterBy) {
            "category" -> {
                selection = "${AbstractProductContract.AbstractProductEntry.CATEGORY} = ?"
                selectionArgs = filter
            }
            "barcodeless" -> {
                selection = "${AbstractProductContract.AbstractProductEntry.BARCODE} = '' "
                selectionArgs = null
            }
        }


        val cursor = db.query(AbstractProductContract.AbstractProductEntry.TABLE_NAME, projection, selection, selectionArgs, null, null, orderBy)

        with (cursor) {
            while(moveToNext()) {
                val productId = getInt(getColumnIndexOrThrow(BaseColumns._ID))
                val barcode = getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.BARCODE))
                val productName = getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NAME))
                val netWeight = getDouble(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.PRODUCT_NET_WEIGHT))
                val productImageHash = getString(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.IMAGE_FILENAME))
                val category = getInt(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.CATEGORY))
                val unit = getInt(getColumnIndexOrThrow(AbstractProductContract.AbstractProductEntry.UNIT))

                val product = AbstractProduct(productId, barcode, productName, netWeight, productImageHash, category, unit)

                abstractProducts.add(product)
            }
        }
        return  abstractProducts
    }

    companion object {
        const val DATABASE_VERSION = 1
        var DATABASE_NAME = "database.db"
    }
}